@@ -24,11 +24,13 @@
 # SOFTWARE.
 
 import socket
+import time
+from enum import IntEnum
 
-from PyQt5.QtGui import *
-from PyQt5.QtCore import *
-from PyQt5.QtWidgets import *
-import PyQt5.QtCore as QtCore
+from PyQt5.QtCore import Qt, pyqtSignal, QThread
+from PyQt5.QtWidgets import (QTreeWidget, QTreeWidgetItem, QMenu, QGridLayout, QComboBox,
+                             QLineEdit, QDialog, QVBoxLayout, QHeaderView, QCheckBox,
+                             QTabWidget, QWidget, QLabel)
 
 from electrum.i18n import _
 from electrum import constants, blockchain
@@ -36,7 +38,7 @@ from electrum.util import print_error
 from electrum.interface import serialize_server, deserialize_server
 from electrum.network import Network
 
-from .util import *
+from .util import Buttons, CloseButton, HelpButton, read_QIcon
 
 protocol_names = ['TCP', 'SSL']
 protocol_letters = 'ts'
@@ -132,6 +134,11 @@ class NodesListWidget(QTreeWidget):
 
 
 class ServerListWidget(QTreeWidget):
+    class Columns(IntEnum):
+        HOST = 0
+        PORT = 1
+
+    SERVER_STR_ROLE = Qt.UserRole + 100
 
     def __init__(self, parent):
         QTreeWidget.__init__(self)
@@ -145,7 +152,7 @@ class ServerListWidget(QTreeWidget):
         if not item:
             return
         menu = QMenu()
-        server = item.data(1, Qt.UserRole)
+        server = item.data(self.Columns.HOST, self.SERVER_STR_ROLE)
         menu.addAction(_("Use as server"), lambda: self.set_server(server))
         menu.exec_(self.viewport().mapToGlobal(position))
 
@@ -176,13 +183,13 @@ class ServerListWidget(QTreeWidget):
             if port:
                 x = QTreeWidgetItem([_host, port])
                 server = serialize_server(_host, port, protocol)
-                x.setData(1, Qt.UserRole, server)
+                x.setData(self.Columns.HOST, self.SERVER_STR_ROLE, server)
                 self.addTopLevelItem(x)
 
         h = self.header()
         h.setStretchLastSection(False)
-        h.setSectionResizeMode(0, QHeaderView.Stretch)
-        h.setSectionResizeMode(1, QHeaderView.ResizeToContents)
+        h.setSectionResizeMode(self.Columns.HOST, QHeaderView.Stretch)
+        h.setSectionResizeMode(self.Columns.PORT, QHeaderView.ResizeToContents)
 
         super().update()
 
@@ -270,7 +277,7 @@ class NetworkChoiceLayout(object):
         self.proxy_password.textEdited.connect(self.proxy_settings_changed)
 
         self.tor_cb = QCheckBox(_("Use Tor Proxy"))
-        self.tor_cb.setIcon(QIcon(":icons/tor_logo.png"))
+        self.tor_cb.setIcon(read_QIcon("tor_logo.png"))
         self.tor_cb.hide()
         self.tor_cb.clicked.connect(self.use_tor_proxy)
 
