@@ -1,5 +1,3 @@
-from PyQt5.QtGui import *
-from PyQt5.QtCore import *
 from PyQt5.QtWidgets import QFileDialog
 
 from electrum.i18n import _
@@ -13,7 +11,7 @@ class ShowQRTextEdit(ButtonsTextEdit):
     def __init__(self, text=None):
         ButtonsTextEdit.__init__(self, text)
         self.setReadOnly(1)
-        icon = ":icons/qrcode_white.png" if ColorScheme.dark_scheme else ":icons/qrcode.png"
+        icon = "qrcode_white.png" if ColorScheme.dark_scheme else "qrcode.png"
         self.addButton(icon, self.qr_show, _("Show as QR code"))
 
         run_hook('show_text_edit', self)
@@ -38,8 +36,8 @@ class ScanQRTextEdit(ButtonsTextEdit, MessageBoxMixin):
         ButtonsTextEdit.__init__(self, text)
         self.allow_multi = allow_multi
         self.setReadOnly(0)
-        self.addButton(":icons/file.png", self.file_input, _("Read file"))
-        icon = ":icons/camera_white.png" if ColorScheme.dark_scheme else ":icons/camera_dark.png"
+        self.addButton("file.png", self.file_input, _("Read file"))
+        icon = "camera_white.png" if ColorScheme.dark_scheme else "camera_dark.png"
         self.addButton(icon, self.qr_input, _("Read QR code"))
         run_hook('scan_text_edit', self)
 
